------------------------------------------------------------------------
-- @class SkyManager
-- Class representing the sky, with time and weather
-- Its skydome is centered and rotates slowly to show alternatively night and day
------------------------------------------------------------------------


class "SkyManager"{
}

function SkyManager:__init__(p_image)
    assert(p_image, "Trying to build skymanager without a skydome image")
    self.skydome    =   p_image
    self.time       =   0
    self.pos        =   {}  --Center at mid-bottom of screen
    self.pos.x      =   love.graphics.getWidth()/2
    self.pos.y      =   love.graphics.getHeight()
    self.offset     =   self.skydome:getWidth()/2
    self.angle      =   0
    self.scale      =   1
    self.timefactor =   0.02
    self.sun        =   Sun()
    self.clouds     =   {}
    cloud_img =   love.graphics.newImage("res/images/level/clouds.png")
    local cloud     =   CloudLayer()
    cloud:setImage( cloud_img )
    table.insert(self.clouds, cloud)

end

function SkyManager:update(dt)
    self.time   =   self.time + dt
    self.angle  =   self.angle+ dt * self.timefactor

    -- Updating sunpos
    local rotation_center   =   vector(love.graphics.getWidth()/2, love.graphics.getHeight())
    local a = self.angle-math.pi/2
    local cd = math.cos ( a )
    local sd = math.sin ( a )
    sunx, suny = rotation_center.x, rotation_center.y
    sunx, suny =  cd* self.sun.distance + sunx, sd*self.sun.distance + suny

    self.sun:update(dt)
    self.sun:setPos(sunx, suny)

    for i, layer in pairs(self.clouds) do
        layer:update(dt)
    end
end

function SkyManager:draw()
    --~ love.graphics.draw(sky, love.graphics.getWidth()/2 - sky:getWidth()/2, love.graphics.getHeight() - sky:getHeight()/2)
    love.graphics.draw( self.skydome, self.pos.x, self.pos.y, self.angle, self.scale, self.scale, self.offset, self.offset )

    --Drawing sun
    self.sun:draw()

    if DEBUG then
        --~ local tmp = 400-love.graphics.getHeight()
        --~ local abs_sunpos        =   vector(self.pos.x + 1000 - self.offset/2, self.pos.y + tmp - self.offset/2)
        --~ local rotation_center   =   vector(love.graphics.getWidth()/2, love.graphics.getHeight())
        --~ local a = self.angle-math.pi/2
        --~ local cd = math.cos ( a )
        --~ local radius = 600
        --~ local sd = math.sin ( a )
        --~ x, y = rotation_center.x, rotation_center.y
        --~ x, y =  cd* radius + x, sd*radius + y
        --~ local v_hook            =   rotation_center
        --~
        --~ local v_hook    =   vector(abs_sunpos.x - rotation_center.x, abs_sunpos.y - rotation_center.y)
        --~ v_hook:rotate_inplace(self.angle-math.pi/2)
        --~ v_hook:normalize_inplace()
        --~ local x, y              =   v_hook.x + rotation_center.x, v_hook.y + rotation_center.y
        --~ print ("sun pos: "..x ..", "..y)

        --~ love.graphics.setColor(255, 80, 80, 255)
        --~ love.graphics.circle( "fill", x, y, 50)
        --~ love.graphics.line( rotation_center.x, rotation_center.y, x, y)
    end
end

function SkyManager:drawClouds()
    for i, layer in pairs(self.clouds) do
        layer:draw()
    end

end

function SkyManager:isNight()
    return (self.angle > 0 and self.angle < math.pi)
end

function SkyManager:isDay()
    return (not self:isNight() )
end
